@@ -70,7 +70,9 @@ from json import JSONDecodeError
 
 import cbor2
 from shapely import geometry
-from json_ntv.namespace import NtvType, Namespace, str_type
+from json_ntv.namespace import NtvType, Namespace, str_type, relative_type, agreg_type
+from observation import Ilist
+
 
 class Ntv(ABC):
     ''' The Ntv class is an abstract class used for all NTV entities.
@@ -160,7 +162,7 @@ class Ntv(ABC):
 
         *Parameters*
 
-        - **value**: value to convert in an Ntv entity
+        - **value**: Ntv value to convert in an Ntv entity
         - **def_type** : NtvType or Namespace (default None) - default type of the NTV entity
         - **def_sep**: ':', '::' or None (default None) - default separator of the Ntv entity'''
         value = Ntv._from_value(value)
@@ -170,7 +172,7 @@ class Ntv(ABC):
         if not sep:
             sep = def_sep
         if isinstance(ntv_value, list) and sep in (None, '::'):
-            def_type = NtvType._agreg_type(str_typ, def_type, False)
+            def_type = agreg_type(str_typ, def_type, False)
             if sep and not def_type:
                 sep = None
             if sep:
@@ -178,17 +180,17 @@ class Ntv(ABC):
             ntv_list = [Ntv.from_obj(val, def_type, sep) for val in ntv_value]
             return NtvList(ntv_list, ntv_name, def_type)
         if sep == ':':
-            ntv_type = NtvType._agreg_type(str_typ, def_type, False)
+            ntv_type = agreg_type(str_typ, def_type, False)
             return NtvSingle(ntv_value, ntv_name, ntv_type,)
         if sep is None and not isinstance(ntv_value, dict):
             is_json = isinstance(value, (int, str, float, bool))
-            ntv_type = NtvType._agreg_type(str_typ, def_type, is_json)
+            ntv_type = agreg_type(str_typ, def_type, is_json)
             return NtvSingle(ntv_value, ntv_name, ntv_type)
         if isinstance(ntv_value, dict) and (sep == '::' or len(ntv_value) != 1 and
                                             sep is None):
             keys = list(ntv_value.keys())
             values = list(ntv_value.values())
-            def_type = NtvType._agreg_type(str_typ, def_type, False)
+            def_type = agreg_type(str_typ, def_type, False)
             if sep and not def_type:
                 sep = None
             if sep:
@@ -196,10 +198,6 @@ class Ntv(ABC):
             ntv_list = [Ntv.from_obj({key: val}, def_type, sep)
                         for key, val in zip(keys, values)]
             return NtvSet(ntv_list, ntv_name, def_type,)
-        if isinstance(ntv_value, dict) and len(ntv_value) == 1 and sep in (None, ':'):
-            ntv_type = NtvType._agreg_type(str_typ, def_type, True)
-            ntv_single = Ntv.from_obj(ntv_value, ntv_type, sep)
-            return NtvSingle(ntv_single, ntv_name, ntv_type)
         raise NtvError('separator ":" is not compatible with value')
 
     @staticmethod
@@ -332,14 +330,14 @@ class Ntv(ABC):
         - **encoded** : boolean (default False) - choice for return format
         (string/bytes if True, dict/list/tuple else)
         - **encode_format**  : string (default 'json')- choice for return format
-        (json, cbor, tuple, obj)
+        (json, cbor, obj)
         - **simpleval** : boolean (default False) - if True, only value (without
         name and type) is included
         '''
         option = {'encoded': False, 'encode_format': 'json',
                   'simpleval': False} | kwargs
         value = self._obj_value(**option)
-        obj_name = self._obj_name(def_type)
+        obj_name = self.obj_name(def_type)
         if option['simpleval']:
             name = ''
         elif option['encode_format'] in ('cbor', 'obj') and not Ntv._is_json_ntv(value):
@@ -391,12 +389,13 @@ class Ntv(ABC):
     def _obj_value(self):
         return ''
 
-    def _obj_name(self, def_type=None):
+    def obj_name(self, def_type=None, string=False):
         '''return the JSON name of the NTV entity (json-ntv format)
 
         *Parameters*
 
-        - **def_typ** : NtvType or Namespace (default None) - type of the parent entity'''
+        - **def_typ** : NtvType or Namespace (default None) - type of the parent entity
+        - **string** : boolean (default False) - If True, return a string else a tuple'''
         if def_type is None:
             def_type = ''
         elif isinstance(def_type, (NtvSingle, Namespace)):
@@ -410,10 +409,12 @@ class Ntv(ABC):
         if not ntv_type:
             json_type = def_type
         else:
-            json_type = NtvType._relative_type(def_type, ntv_type)
+            json_type = relative_type(def_type, ntv_type)
         json_sep = ''
         if json_type or (len(self.ntv_value) == 1 and self.__class__.__name__ == 'NtvSet'):
             json_sep = '::'
+        if string:
+            return json_name + json_sep + json_type
         return (json_name, json_sep, json_type)
 
     @staticmethod
@@ -428,41 +429,42 @@ class Ntv(ABC):
         if isinstance(json_value, dict) and len(json_value) == 1:
             json_name = list(json_value.keys())[0]
             val = json_value[json_name]
-            nam, typ, sep = Ntv._from_obj_name(json_name)
+            nam, typ, sep = Ntv.from_obj_name(json_name)
             return (nam, typ, val, sep)
         return(*Ntv._cast(json_value), ':')
 
     @staticmethod
     def _cast(data):
         '''return (name, type, value) of the data'''
-        dic_geo_cl = {'point': 'point', 'multipoint': 'multipoint', 'linestring': 'line',
-                      'multilinestring': 'multiline', 'polygon': 'polygon',
-                      'multipolygon': 'multipolygon'}
-        #dic_connec = {'series': 'SeriesConnec', 'dataframe': 'DataFrameConnec'}
+        dic_geo_cl = {'Point': 'point', 'MultiPoint': 'multipoint', 'LineString': 'line',
+                      'MultiLineString': 'multiline', 'Polygon': 'polygon',
+                      'MultiPolygon': 'multipolygon'}
         dic_connec = NtvConnector.dic_connec()
-        clas = data.__class__.__name__.lower()
+        clas = data.__class__.__name__
         match clas:
-            case 'date':
-                return (None, 'date', data.isoformat())
-            case 'time':
-                return (None, 'time', data.isoformat())
-            case 'datetime':
-                return (None, 'datetime', data.isoformat())
-            case 'point' | 'multipoint' | 'linestring' | 'multilinestring' | \
-                    'polygon' | 'multipolygon':
-                return (None, dic_geo_cl[data.__class__.__name__.lower()],
+            case 'date' | 'time' | 'datetime':
+                return (None, clas, data.isoformat())
+            case 'Point' | 'MultiPoint' | 'LineString' | 'MultiLineString' | \
+                    'Polygon' | 'MultiPolygon':
+                return (None, dic_geo_cl[data.__class__.__name__],
                         Ntv._listed(data.__geo_interface__['coordinates']))
+            case 'NtvSingle' | 'NtvSet' | 'NtvList':
+                return (None, 'ntv', data.to_obj())
+            case 'Ilist':
+                return (None, 'tab', data.to_obj())
             case _:
-                connector = None
+                connec = None
                 if clas in dic_connec and dic_connec[clas] in NtvConnector.connector():
-                    connector = NtvConnector.connector()[dic_connec[clas]]
-                if connector:
-                    return connector.to_ntv(data)
+                    connec = NtvConnector.connector()[dic_connec[clas]]
+                if connec:
+                    return connec.to_ntv(data)
                 raise NtvError('connector is not defined to NTV entity')
         return (None, None, None)
 
     def _uncast(self, **option):
         '''return object from ntv_value'''
+        dic_fct = {'date': datetime.date.fromisoformat, 'time': datetime.time.fromisoformat,
+                   'datetime': datetime.datetime.fromisoformat}
         dic_geo = {'point': 'point', 'multipoint': 'multipoint', 'line': 'linestring',
                    'multiline': 'multilinestring', 'polygon': 'polygon',
                    'multipolygon': 'multipolygon'}
@@ -470,41 +472,37 @@ class Ntv(ABC):
                     'multiline': False, 'polygon': False, 'multipolygon': False,
                     'date': True, 'time': False, 'datetime': True}
         dic_obj = {'tab': 'Ilist', 'other': None}
+        type_n = self.ntv_type.name
         if 'dicobj' in option:
             dic_obj |= option['dicobj']
-        obj = True
-        if option['encode_format'] == 'cbor':
-            obj = False
-            if self.ntv_type and self.ntv_type.name in dic_cbor:
-                obj = dic_cbor[self.ntv_type.name]
+        obj = not option['encode_format'] == 'cbor' or \
+            (self.ntv_type and type_n in dic_cbor and dic_cbor[type_n])
         if self.ntv_type is None or not obj:
             return self.ntv_value
-        match self.ntv_type.name:
-            case 'date':
-                return datetime.date.fromisoformat(self.ntv_value)
-            case 'time':
-                return datetime.time.fromisoformat(self.ntv_value)
-            case 'datetime':
-                return datetime.datetime.fromisoformat(self.ntv_value)
-            case 'point' | 'multipoint' | 'line' | \
-                 'multiline' | 'polygon' | 'multipolygon':
-                return geometry.shape({"type": dic_geo[self.ntv_type.name],
-                                      "coordinates": self.ntv_value})
-            case _:
-                connector = None
-                if self.ntv_type.name in dic_obj and \
-                        dic_obj[self.ntv_type.name] in NtvConnector.connector():
-                    connector = NtvConnector.connector(
-                    )[dic_obj[self.ntv_type.name]]
-                elif dic_obj['other'] in NtvConnector.connector():
-                    connector = NtvConnector.connector()['other']
-                if connector:
-                    return connector.from_ntv(self.ntv_value)
-                return self.ntv_value
+        if type_n in dic_fct:
+            return dic_fct[type_n](self.ntv_value)
+        if type_n == 'ntv':
+            return Ntv.obj(self.ntv_value)
+        if type_n in dic_geo:
+            return geometry.shape({"type": dic_geo[type_n],
+                                   "coordinates": self.ntv_value})
+        if type_n == 'tab' and dic_obj[type_n] == 'Ilist':
+            if isinstance(self.ntv_value, list):
+                return Ilist.obj(self.ntv_value)
+            return Ilist.dic(self.ntv_value)
+        connec = None
+        if type_n in dic_obj and \
+                dic_obj[type_n] in NtvConnector.connector():
+            connec = NtvConnector.connector()[dic_obj[type_n]]
+        elif dic_obj['other'] in NtvConnector.connector():
+            connec = NtvConnector.connector()['other']
+        if connec:
+            return connec.from_ntv(self.ntv_value)
+        return self.ntv_value
 
     @staticmethod
-    def _from_obj_name(string):
-        '''return a tuple with name, type ans separator from string'''
+    def from_obj_name(string):
+        '''return a tuple with name, type and separator from string'''
         if not isinstance(string, str):
             raise NtvError('a json-name have to be str')
         if string == '':
@@ -535,6 +533,7 @@ class Ntv(ABC):
         '''transform a tuple of tuple in a list of list'''
         return [val if not isinstance(val, tuple) else Ntv._listed(val) for val in idx]
 
+
 class NtvSingle(Ntv):
     ''' An NTV-single entity is a Ntv entity not composed with other entities.
 
@@ -572,7 +571,8 @@ class NtvSingle(Ntv):
         - **ntv_type**: String (default None) - type of the entity
         - **value**: value of the entity
         '''
-        is_json_ntv = Ntv._is_json_ntv(value) or isinstance(value, NtvSingle)
+        is_json_ntv = Ntv._is_json_ntv(
+            value)  # or isinstance(value, NtvSingle)
         if not ntv_type and is_json_ntv:
             ntv_type = 'json'
         if not ntv_type and not is_json_ntv:
@@ -595,30 +595,25 @@ class NtvSingle(Ntv):
         '''return the Json format of the ntv_value'''
         option = {'encoded': False, 'encode_format': 'json',
                   'simpleval': False} | kwargs
-        if isinstance(self.ntv_value, NtvSingle):
-            def_type = ''
-            if self.ntv_type:
-                def_type = self.ntv_type.long_name
-            option2 = option | {'encoded': False}
-            return self.ntv_value.to_obj(def_type=def_type, **option2)
         if option['encode_format'] in ('json', 'tuple'):
             return self.ntv_value
         return Ntv._uncast(self, **option)
 
-    def _obj_name(self, def_type=None):
+    def obj_name(self, def_type=None, string=False):
         '''return the JSON name of the NTV entity (json-ntv format)
 
         *Parameters*
 
-        - **def_typ** : NtvType or Namespace (default None) - type of the parent entity'''
+        - **def_typ** : NtvType or Namespace (default None) - type of the parent entity
+        - **string** : boolean (default False) - If True, return a string else a tuple'''
         if def_type is None:
             def_type = ''
-        elif isinstance(def_type, (NtvSingle, Namespace)):
+        elif isinstance(def_type, (NtvType, Namespace)):
             def_type = def_type.long_name
         json_name = ''
         if self.ntv_name:
             json_name = self.ntv_name
-        json_type = NtvType._relative_type(def_type, self.ntv_type.long_name)
+        json_type = relative_type(def_type, self.ntv_type.long_name)
         json_sep = ''
         if json_type:
             json_sep = ':'
@@ -627,6 +622,8 @@ class NtvSingle(Ntv):
             if not isinstance(self.ntv_value, list) and \
                not (isinstance(self.ntv_value, dict) and len(self.ntv_value) != 1):
                 json_sep = ''
+        if string:
+            return json_name + json_sep + json_type
         return (json_name, json_sep, json_type)
 
 
