@@ -9,9 +9,14 @@ The `NTV.test_ntv` module contains the unit tests (class unittest) for the
 """
 import unittest
 import datetime
-from json_ntv import NtvSingle, NtvList, NtvSet, Ntv, NtvError, NtvType
+import csv
+
+from json_ntv import NtvSingle, NtvList, NtvSet, Ntv, NtvError, NtvType, from_csv, to_csv, agreg_type
 from shapely import geometry
 
+from ntv_connector import to_csv, from_csv
+from observation import Ilist
+
 
 class Test_Ntv_creation(unittest.TestCase):
 
@@ -37,7 +42,7 @@ class Test_Ntv_creation(unittest.TestCase):
                     ]
         for data in list_obj:
             ntv = Ntv.obj(data[1])
-            self.assertEqual(ntv._obj_name(data[0]), data[2])
+            self.assertEqual(ntv.obj_name(data[0]), data[2])
 
     def test_agreg_type(self):
         list_type = [[[None, None, True], 'json'],
@@ -59,10 +64,10 @@ class Test_Ntv_creation(unittest.TestCase):
         for typ in list_type:
             # print(typ[0])
             if typ[0] == [None, None, False]:
-                self.assertEqual(NtvType._agreg_type(
+                self.assertEqual(agreg_type(
                     typ[0][0], typ[0][1], typ[0][2]), typ[1])
             else:
-                self.assertEqual(NtvType._agreg_type(
+                self.assertEqual(agreg_type(
                     typ[0][0], typ[0][1], typ[0][2]).long_name, typ[1])
 
     def test_from_obj_repr(self):
@@ -82,11 +87,12 @@ class Test_Ntv_creation(unittest.TestCase):
                          '{"l": [{"l": ["v", "v"]}, {"l": ["v", "v"]}]}'],
                      [{"::point": [[1, 2], [3, 4]]}, '{"lT": ["vT", "vT"]}'],
                      [{"a": 2}, '"vN"'],
-                     [{"truc": {"a": 2}}, '{"vN": "vN"}'],
+                     #[{"truc": {"a": 2}}, '{"vN": "vN"}'],
                      [{":point": {"a": 2}}, '"vT"'],
                      [{"truc:": {"a": 2}}, '"vN"'],
                      [{":": {"a": 2}}, '"v"']]
         for test in list_repr:
+            #print(test)
             self.assertEqual(repr(Ntv.from_obj(test[0])), test[1])
 
     def test_from_obj_ko(self):
@@ -128,15 +134,21 @@ class Test_Ntv_creation(unittest.TestCase):
                    '1NtvSingle': 1,
                    '2NtvSingle': 'test',
                    '3NtvSingle': {'single': 1},
-                   '4NtvSingle': {'ntv1': {'ntv2': 2}},
+                   '4NtvSingle': {'ntv1:ntv': {'ntv2': 2}},
                    '5NtvSingle': {'ntv1:fr.reg': {'ntv2:fr.BAN.lon': 2}},
                    '6NtvSingle': {'ntv1': True},
                    '7NtvSingle': True,
                    '8NtvSingle': {'ntv1:dat': [1, 2]},
                    '9NtvSingle': {'ntv1': datetime.date(2021, 2, 1)},
+                   'dNtvSingle': datetime.date(2021, 2, 1),
                    'aNtvSingle': '{ner',
                    'bNtvSingle': {':$point': {'a': [1, 2], 'b': [3, 4]}},
                    'cNtvSingle': {':': [{'paris': [2.1, 40.3]}, {'lyon': [2.1, 40.3]}]},
+                   'eNtvSingle': {':': NtvSingle(1,'test')},
+                   'hNtvSingle': {'set:': NtvSet([{'l1':21}, {'l2': [2,3]}])},
+                   'fNtvSingle': {'set:': NtvSet([{'l1':21}, {'l2': datetime.date(2021, 2, 1)}])},
+                   'gNtvSingle': {'lis:': NtvList([{'l1':21}, {'l2': datetime.date(2021, 2, 1)}])},
+                   'iNtvSingle': {'lis:': NtvList([1,2,3])},
                    '1NtvList': [],
                    '2NtvList': [[4, [5, 6]], {'heure': [21, 22]}],
                    '3NtvList': [[4, 5], {'heure': 21}],
@@ -166,8 +178,9 @@ class Test_Ntv_creation(unittest.TestCase):
             ntv2 = Ntv.obj(nstr)
             self.assertEqual(ntv, ntv2)
             #print('ap', nstr, typ)
-            if not typ in ['9NtvList', 'aNtvList', 'bNtvList', '9NtvSingle',
-                           '4NtvSet', '5NtvSet']:
+            if not typ in ['9NtvList', 'aNtvList', 'bNtvList', '9NtvSingle', 'eNtvSingle',
+                           'dNtvSingle', 'fNtvSingle', 'gNtvSingle', 'hNtvSingle',
+                           'iNtvSingle', '4NtvSet', '5NtvSet']:
                 self.assertEqual(nstr, ntv.to_obj())
             if typ == 'bNtvList':
                 self.assertEqual(nstr['::'], ntv.to_obj())
@@ -201,19 +214,57 @@ class Test_Ntv_creation(unittest.TestCase):
         for test in list_test:
             # print(test[1])
             self.assertEqual(Ntv.from_obj(
-                test[1]).ntv_value[0]._obj_name(), test[0])
+                test[1]).ntv_value[0].obj_name(), test[0])
             self.assertEqual(
-                Ntv.obj(test[1]).ntv_value[0]._obj_name(), test[0])
+                Ntv.obj(test[1]).ntv_value[0].obj_name(), test[0])
 
     def test_to_obj(self):
         nstr = {'cities': [{'paris': [2.1, 40.3]}, {'lyon': [2.1, 40.3]}]}
         self.assertEqual(Ntv.from_obj(nstr).to_obj(
             simpleval=True), [[2.1, 40.3], [2.1, 40.3]])
-        sing = Ntv.from_obj({'ntv1': {'ntv2': 2}})
-        self.assertTrue(isinstance(sing.ntv_value, NtvSingle))
 
     def test_pandas(self):
-        pass
+        field = Ntv.obj({':field': 
+                     {'dates::datetime': ['1964-01-01', '1985-02-05', '2022-01-21']}})
+        tab   = Ntv.obj({':tab'  :
+                     {'index':           [1, 2, 3],
+                      'dates::datetime': ['1964-01-01', '1985-02-05', '2022-01-21'], 
+                      'value':           [10, 20, 30],
+                      'value32::int32':  [10, 20, 30],
+                      'coord::point':    [[1,2], [3,4], [5,6]],
+                      'names::string':   ['john', 'eric', 'judith']}})
+        sr = field.to_obj(encode_format='obj', dicobj={'field': 'SeriesConnec'})
+        df = tab.to_obj  (encode_format='obj', dicobj={'tab': 'DataFrameConnec'})
+        self.assertTrue(df.equals(Ntv.obj(df).to_obj(encode_format='obj', dicobj={'tab': 'DataFrameConnec'})))
+        self.assertTrue(sr.equals(Ntv.obj(sr).to_obj(encode_format='obj', dicobj={'field': 'SeriesConnec'})))
 
+    def test_observation(self):
+        field = Ntv.obj({':field': 
+                     {'dates::datetime': ['1964-01-01', '1985-02-05', '2022-01-21']}})
+        tab = { 'index':           [1, 2, 3],
+                'dates::datetime': ['1964-01-01', '1985-02-05', '2022-01-21'], 
+                'value':           [10, 20, 30],
+                'value32::int32':  [10, 20, 30],
+                'coord::point':    [[1,2], [3,4], [5,6]],
+                'names::string':   ['john', 'eric', 'judith']}
+        il   = Ilist.dic(tab)
+        ntv = Ntv.obj( il)
+        self.assertEqual(il, ntv.to_obj(encode_format='obj'))
+        tab_ntv = Ntv.obj({':tab'  : tab})
+        il = tab_ntv.to_obj  (encode_format='obj')
+        self.assertEqual(il, Ntv.obj(il).to_obj(encode_format='obj'))
+
+                          
+    def test_csv(self):
+        tab   = Ntv.obj({':tab'  :
+                     {'index':           [1, 2, 3],
+                      'dates::datetime': ['1964-01-01', '1985-02-05', '2022-01-21'], 
+                      'value':           [10, 20, 30],
+                      'value32::int32':  [10, 20, 30],
+                      'coord::point':    [[1,2], [3,4], [5,6]],
+                      'names::string':   ['john', 'eric', 'judith']}})        
+        self.assertEqual(tab, from_csv(to_csv('test.csv', tab)))
+        self.assertEqual(tab, from_csv(to_csv('test.csv', tab, quoting=csv.QUOTE_ALL)))
+        
 if __name__ == '__main__':
     unittest.main(verbosity=2)
